# Macros from fmpz_mat.h
# See https://github.com/flintlib/flint/issues/1529

from .types cimport *

cdef extern from "flint_wrap.h":
    fmpz * fmpz_mat_entry(fmpz_mat_t mat, slong i, slong j)
    slong fmpz_mat_nrows(fmpz_mat_t)
    slong fmpz_mat_ncols(fmpz_mat_t)



import os
import multiprocessing
from setuptools.command.build_ext import build_ext


class sage_build_ext_minimal(build_ext):
    """
    In contrast to :func:`~sage_setup.sage_build_ext.sage_build_ext`, this build extension is designed
    to be used in combination with Cython's cythonize function.
    Thus, we only take care of some options and letting Cython do the main work.
    """

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.parallel = self.get_default_number_build_jobs()

    @staticmethod
    def get_default_number_build_jobs() -> int:
        """
        Get number of parallel build jobs used by default, i.e. unless explicitly
        set by the --parallel command line argument of setup.py.

        First, the environment variable ``SAGE_NUM_THREADS`` is checked.
        If that is unset, return the number of processors on the system,
        with a maximum of 10 (to prevent overloading the system if there a lot of CPUs).

        OUTPUT:
            number of parallel jobs that should be run
        """
        try:
            cpu_count = len(os.sched_getaffinity(0))
        except AttributeError:
            cpu_count = multiprocessing.cpu_count()
        cpu_count = min(cpu_count, 10)
        return int(os.environ.get("SAGE_NUM_THREADS", cpu_count))



#!/usr/bin/env python3
# Usage: python -m sage_setup.autogen.interpreters <output_dir>

import argparse

from internal import rebuild

parser = argparse.ArgumentParser()
parser.add_argument("output_dir", help="Output directory")
args = parser.parse_args()

rebuild(args.output_dir)



#########################################################
### Install Jupyter kernel spec
#########################################################

import os
import time

# Import setuptools before importing distutils, so that setuptools
# can replace distutils by its own vendored copy.
import setuptools

from distutils import log
from distutils.command.install import install
from setuptools.command.develop import develop


class install_kernel_spec_mixin:

    def install_kernel_spec(self):
        """
        Install the Jupyter kernel spec.

        .. NOTE::

            The files are generated, not copied. Therefore, we cannot
            use ``data_files`` for this.
        """
        from sage.repl.ipython_kernel.install import SageKernelSpec
        # Jupyter packages typically use the data_files option to
        # setup() to install kernels and nbextensions. So we should use
        # the install_data directory for installing our Jupyter files.
        SageKernelSpec.update(prefix=self.install_data)


class sage_install(install, install_kernel_spec_mixin):

    def run(self):
        install.run(self)
        self.install_kernel_spec()


class sage_develop(develop, install_kernel_spec_mixin):

    def run(self):
        develop.run(self)
        if not self.uninstall:
            self.install_kernel_spec()
